#!/usr/bin/python
#   coding: utf-8

import re
import time
import cStringIO
import xml.etree.cElementTree as etree
import pad
import pad.tree

from pad.logger import Logger
logger = Logger()

padns='{http://pad.colourcountry.net/xmlns/model}'

def new(className, objargs, **aargs):
  ns = aargs.get( 'namespace', globals() )
  if '.' in className:
    theModule, theRest = className.split('.',1)
    return new(theRest, objargs, namespace = ns[theModule].__dict__)
  logger.log( 4, 'New ',className,' ',objargs )
  return ns[className](**objargs)

def getDict( el ):
  args = {}
  for child in el:
    if child.tag == padns+'string':
      args[child.get('name')] = child.get('value', '')
    elif child.tag == padns+'list':
      args[child.get('name')] = getList( child )
    else:
      raise ValueError( 'Unknown tag '+child.tag )
  return args

def getList( el ):
  args = []
  for child in el:
    if child.tag == padns+'string':
      args.append( child.get('value', '') )
    elif child.tag == padns+'list':
      args.append( getList( child ) )
    else:
      raise ValueError( 'Unknown tag '+child.tag )
  return args



class Model:
  @classmethod
  def getStyleDef( self, name ):
    return {}

  def __init__( self, template=None, name='Unnamed' ):
    if template:
      self.template = template
    else:
      self.template = '''<!DOCTYPE null SYSTEM "http://wherever.org.uk/" ><null />'''
    self.name = name

  def getTemplate( self, ns, path, props ):
    tree = pad.tree.Tree( self.template, time.time(), ns, path )
    tree.replaceValues( props )
    return tree

  def getContent( self, el, styleName, props, content=None ):
    try:
      theStyle = self.styles[styleName]
    except KeyError:
      logger.log(KeyError, 'Requested style ', styleName,' is not supported by the model ',self.name,': options are ',self.styles.keys() )
    return theStyle.apply( el, props, content )

  def getProperties( self, el ):
    props = {}
    self.setter.getProperties( el, props )
    return props



class XmlModel(Model):

  DEFAULT = '''<pad:model xmlns:pad="http://pad.colourcountry.net/xmlns/model">

  <pad:elements>
    <pad:element>
      <pad:mode name="edit">
      <pad:style>
        <pad:editor name="pad.types.structure.Flag" />
      </pad:style>
      </pad:mode>
    </pad:element>
  </pad:elements>

</pad:model>
'''



  def __init__( self, modelDef, name='Unnamed' ):

    self.name = name

    if isinstance( modelDef, str ):

      content = file( filename, 'r' ).read()
      parser = etree.XMLParser()
      parser.feed( content )
      tree = parser.close()
    elif hasattr( modelDef, "tree" ):
      tree = modelDef.tree
    else:
      raise ValueError( "Bad model definition "+repr(modelDef) )

    logger.log(3, 'Creating model for ', tree.tag )
    assert tree.tag.startswith(padns)

    editModes = {}
    validModes = {}
    setterDef = {}

    elementsNode = tree.find( padns+'elements')
    if elementsNode is not None:
     for element in elementsNode.findall( padns+'element' ):
      elementName = element.get( 'name', None )
      logger.log(4, 'Building element ', elementName, '...' )

      setter = element.find( padns+'setter' )
      if setter:
        args = getDict( setter )
        setterDef[elementName] = new( setter.get('name'), args )

      for mode in element.findall( padns+'mode' ):
        modeName = mode.get( 'name', None )
        logger.log(4, 'Adding mode ', modeName )

        if not editModes.has_key( modeName ):
          editModes[modeName] = {}
        if not editModes[modeName].has_key( elementName ):
          editModes[modeName][elementName] = []
        editStyles = []

        if not validModes.has_key( modeName ):
          validModes[modeName] = {}
        if not validModes[modeName].has_key( elementName ):
          validModes[modeName][elementName] = []
        validStyles = []

        for style in mode.findall( padns+'style' ):
          tests = []

          for test in style.findall( padns+'test' ):
            tests.append( test.get( 'property', None ) )
            tests.append( test.get( 'value', None ) )

          editor = style.find( padns+'editor' )

          if editor is not None:
            editStyles.append(tests)
            args = getDict( editor )
            logger.log(4, 'Editor for ', elementName,' is ',editor.get('name') )
            editStyles.append( new( editor.get('name'), args ) )

          validator = style.find( padns+'validator' )

          if validator is not None:
            validStyles.append(tests)
            args = getDict( validator )
            validStyles.append( new( validator.get('name'), args ) )

        editModes[modeName][elementName] = editStyles
        validModes[modeName][elementName] = validStyles
        logger.log(4, 'Added mode ', modeName )
      logger.log(4, 'Finished element ', elementName, '.' )

    self.styles = {}
    for mode in editModes.keys():
      logger.log(4, mode, ' (edit) = ', editModes[mode])
      self.styles[mode] = pad.style.Style( editModes[mode] )

    self.validators = {}
    for mode in validModes.keys():
      logger.log(4, mode, ' (valid) = ', validModes[mode])
      self.validators[mode] = pad.validator.ByTag( validModes[mode] )

    self.setter = pad.setter.Setter( setterDef )

    
    
    templates = tree.find( padns+'template' )
    
    if templates:
      crappyApi = cStringIO.StringIO()
      etree.ElementTree( templates[0] ).write( crappyApi )
      crappyApi.reset()
      self.template = crappyApi.read()
    else:
      self.template = None

    logger.log(4, 'template = ', self.template)



class ModelRegister:
  def __init__( self, storage, ns ):
    self.register = {}
    self.storage = storage
    self.ns = ns

  def __getitem__( self, key ):
    if not self.register.has_key(key):
      if not self.findModel(key):
        return None
    return self.register[key]

  def findModel( self, key ):
     logger.log(3, 'Fetching new model for ',key)
     try:
       modelDef = self.storage.get( self.ns, key )
       if modelDef is None:
         return None
     except ValueError, e:
       logger.log(3, e)
       raise KeyError
     model = XmlModel( modelDef, key )
     self.register[key] = model
     return model

  def getByPath( self, ns, path ):
    result = None
    while result is None:
      uri = 'pad://'+ns+'/'+path
      result = self[uri]
      if result is None:
        logger.log(3,'No model for path ',ns,':',path)
        try:
          rindex = path.rindex('/')
        except ValueError:
          logger.log(0,'No model, fallback failed')
          raise ValueError
        path = path[:rindex]
      else:
        logger.log(3, 'Model for path ',ns,':',path,' is ',uri)
        return result



if __name__ == "__main__":
  model = XmlModel('pagedef.xml')


