#!/usr/bin/python
#   coding: utf-8

from pad.type import Type
from pad.helper import *
from pad.logger import Logger
logger = Logger()

class Editor(Type):
  typeName = 'pad.types.active.ckeditor.Editor'

  emptyText = unichr(0x270e)+" click to add content"

  def getContent( self, el, style, props, content ):
    href = Type.getActiveHref( props )
    location = Type.getLocation( props )

    if (el.text and el.text.strip()) or len(el)>0:
      theText = formatChildren( el )
      theClass = "_text"
    else:
      theText = Editor.emptyText
      theClass = "_editlink"
 
    theTag = makeattr(el.tag)

    theStyle = self.options.get( 'style', '' )

    # &#x238c; is the real undo symbol

    content += u'''
<div class="'''+theTag+u'''"><div class="_">
  <form name="form_'''+location+u'''">
    <div style="'''+theStyle+'''" class="'''+theClass+'''" onclick="ckeditor_editor_activate(this)">'''

    content += theText

    content += u'''</div>
    <div class="_edit" style="display: none">
      <div>
        <button type="button" name="submit">&#x2714;</button>
        <button type="button" name="cancel">&#x21b6;</button>
      </div>
    </div>
    <input type="hidden" name="location" value="'''+location+'''" />
    <input type="hidden" name="action" value="replace" />
    <input type="hidden" name="a" value="" />
    <input type="hidden" name="b" value="" class="_full" style="'''+theStyle+'''"/>
  </form>
</div>
'''

    content += u'\n</div><!-- end '+el.tag+u' -->\n'

  def getScript( self, el, props ):
    return ['''
if (undefined===window.ckeditor_editor) ckeditor_editor = null;
if (undefined===window.ckeditor_editor_parent) ckeditor_editor_parent = null;
''','''
ckeditor_editor_deactivate = function( p ) {
  return function() {
    var node = p.parentNode;
    var newValue = node.elements['b'].value;
    if (newValue == "") {
      p.innerHTML = "'''+Editor.emptyText+'''";
      p.setAttribute("class","_editlink");
    } else {
      p.innerHTML = newValue;
      p.setAttribute("class","_text");
    }
    p.style.display="block";
    node.getElementsByTagName('div')[1].style.display="none";

    p.style.display="block";
    window.pad_activeNode = null;
    window.pad_forceDecision = null;
  }
}
''','''
ckeditor_editor_warn = function() {
  var node = window.pad_activeNode;
}
''','''
ckeditor_editor_activate = function( p ) {
  if (window.pad_forceDecision != null) {
    window.pad_forceDecision();
    return false;
  }

  var node = p.parentNode;

  window.pad_activeNode = node;

  editDiv = node.getElementsByTagName('div')[1];


  if (!ckeditor_editor) {
    var editContainer = document.createElement('div');
    ckeditor_editor = CKEDITOR.appendTo( editContainer );
    ckeditor_editor_container = editContainer;
    editDiv.insertBefore( editContainer, editDiv.firstChild );
  } else {
    var editContainer = ckeditor_editor_container.parentNode.removeChild( ckeditor_editor_container );
    editDiv.insertBefore( editContainer, editDiv.firstChild );
  }



  ckeditor_editor_container = editContainer;
  editDiv.style.display="block";
  p.style.display="none";

  window.pad_touch( node );

  window.pad_deactivate();
  window.pad_deactivate = ckeditor_editor_deactivate( p );


  window.pad_forceDecision = ckeditor_editor_warn;

  node.elements['cancel'].onclick = function( e ) {
    node.elements['b'].value = node.elements['a'].value;
    ckeditor_editor_deactivate( p )();
    window.pad_deactivate = window.pad_doNothing;
  }

  node.elements['submit'].onclick = function( e ) {
    node.elements['b'].value = ckeditor_editor.getData();
    window.pad_submit( e.target );
  }

  if (p.getAttribute("class") == "_editlink") {
    node.elements['a'].value = "";
    ckeditor_editor.setData( "" );
  } else {
    node.elements['a'].value = p.innerHTML;
    ckeditor_editor.setData( p.innerHTML );
  }


}
''']

