#!/usr/bin/python
#   coding: utf-8

from pad.type import Type
from pad.helper import *
from pad.logger import Logger
logger = Logger()

class Block(Type):
  typeName = 'pad.types.active.text.Block'

  emptyContent = unichr(0x270e)+" click to add text"

  def getContent( self, el, style, props, content ):
    href = Type.getActiveHref( props )
    location = Type.getLocation( props )

    if el.text and el.text.strip():
      theText = maketext( el.text )
      theClass = "_text"
    else:
      theText = Block.emptyContent;
      theClass = "_editlink"
 
    theTag = makeattr(el.tag)

    theStyle = self.options.get( 'style', '' )

    # &#x238c; is the real undo symbol

    content += u'''
<div class="'''+theTag+u'''"><div class="_">
  <form name="form_'''+location+u'''">
    <a style="display: block; '''+theStyle+'''" class="'''+theClass+'''" onclick="block_activate(this)">'''+theText+'''</a>
    <input type="hidden" name="location" value="'''+location+'''" />
    <input type="hidden" name="action" value="replace" />
    <input type="hidden" name="a" value="" />
    <input type="hidden" name="b" value="" class="_full" style="'''+theStyle+'''"/>
    <button type="button" style="display:none" name="submit">&#x2714;</button>
    <button type="button" style="display:none" name="cancel">&#x21b6;</button>
  </form>
</div>
'''

    content += style.apply( el, props )
    content += u'\n</div><!-- end '+el.tag+u' -->\n'

  def getScript( self, el, props ):
    return [
'''
block_deactivate = function( p ) {
  return function() {
    var node = p.parentNode;
    var newValue = node.elements['b'].value;
    if (newValue == "") {
      p.firstChild.nodeValue = "'''+Block.emptyContent+'''";
      p.setAttribute("class","_editlink");
    } else {
      p.firstChild.nodeValue = newValue;
      p.setAttribute("class","_text");
    }
    node.elements['b'].type="hidden";
    node.elements['submit'].style.display="none";
    node.elements['cancel'].style.display="none";
    p.style.display="block";
    window.pad_activeNode = null;
    window.pad_forceDecision = null;
  }
}
''','''
block_warn = function() {
  var node = window.pad_activeNode;
}
''','''
block_activate = function( p ) {

  if (window.pad_forceDecision != null) {
    window.pad_forceDecision();
    return false;
  }

  var node = p.parentNode;
  window.pad_activeNode = node;

  var theText = "";
  if (p.getAttribute("class") != "_editlink") {
    theText = p.firstChild.nodeValue;
  }

  window.pad_touch( node );

  window.pad_deactivate();
  window.pad_deactivate = block_deactivate( p );

  if (node.elements['a'].value == '') {
    node.elements['a'].value = theText;
  }

  node.elements['b'].value = theText;
  node.elements['b'].type = "text";
  node.elements['b'].focus();
  node.elements['b'].setSelectionRange( theText.length, theText.length );

  p.style.display = "none";

  node.elements['b'].onkeydown = function( e ) {
    var node = e.target.parentNode;
    node.elements['submit'].style.display = "inline";
    node.elements['cancel'].style.display = "inline";
    if ( e.keyCode == 13 ) {
      window.pad_submit( e.target );
      return false;
    }
    window.pad_forceDecision = block_warn;
  }

  node.elements['cancel'].onclick = function( e ) {
    var node = e.target.parentNode;
    node.elements['b'].value = node.elements['a'].value;
    block_deactivate( p )();
    window.pad_deactivate = window.pad_doNothing;
  }

  node.elements['submit'].onclick = function( e ) {
    window.pad_submit( e.target );
  }

}
''']


class BlockSplit(Type):
  typeName = 'pad.types.activetext.BlockSplit'

  emptyContent = unichr(0x270e)+" click to add text"

  def getContent( self, el, style, props, content ):
    href = Type.getActiveHref( props )
    location = Type.getLocation( props )

    if el.text and el.text.strip():
      theText = maketext( el.text )
      theClass = "_text"
    else:
      theText = Block.emptyContent;
      theClass = "_editlink"
 
    theTag = makeattr(el.tag)

    content += u'''
<div class="'''+theTag+u'''"><div class="_">
  <form name="form_'''+location+u'''" onclick="blocksplit_activate(this)" action="'''+href+u'''" method="GET"><a class="'''+theClass+'''">'''+theText+'''</a>
    <input type="hidden" name="location" value="'''+location+'''" />

    <input type="hidden" name="action" value="addbefore" />
    <input type="hidden" name="a" value="'''+theTag+'''" />
    <input type="hidden" name="b" value="" class="_full" />

    <input type="hidden" name="action_2" value="addbefore" />
    <input type="hidden" name="a_2" value="'''+theTag+'''" />
    <input type="hidden" name="b_2" value="" />

    <input type="hidden" name="action_3" value="delete" />
    <input type="hidden" name="a_3" value="'''+theTag+'''" />

    <input type="hidden" name="action_4" value="editafter" />
    <input type="hidden" name="a_4" value="'''+theTag+'''" />
  </form>
</div>
'''

    content += style.apply( el, props )
    content += u'\n</div><!-- end '+el.tag+u' -->\n'

  def getScript( self, el, props ):
    return [
'''

blocksplit_key = function( form, event ) {
  if (event.which == 13) {
    alert("Oi!");
    return false;
  } else {
    return true;
  }
}

blocksplit_deactivate = function( node ) {
  return function() {
    var newValue = node.elements['b'].value;
    var p = node.firstChild;
    if (newValue == "") {
      p.firstChild.nodeValue = "'''+Block.emptyContent+'''";
      p.setAttribute("class","_editlink");
    } else {
      p.firstChild.nodeValue = newValue;
      p.setAttribute("class","_text");
    }
    node.elements['b'].type="hidden";
    p.style.display="block";
  }
}
''','''
blocksplit_key = function( form ) {
}
''','''
blocksplit_activate = function( node ) {

  var p = node.firstChild;
  var theText = "";
  if (p.getAttribute("class") != "_editlink") {
    theText = p.firstChild.nodeValue;
  }

  window.pad_touch( node );

  window.pad_deactivate();
  window.pad_deactivate = blocksplit_deactivate( node );

  node.elements['b'].value = theText;
  node.elements['b'].type = "text";
  p.style.display = "none";

}
''']
