#!/usr/bin/python
#   coding: utf-8

from pad.type import Type
from pad.helper import *
from pad.logger import Logger
logger = Logger()

class Block(Type):
  def getContent( self, el, style, props, content ):
    href = Type.getHref( props )

    content += u'<div class="'+makeattr(el.tag)+u'"><div class="_">'
    if el.text and el.text.strip():
      content += u'<a href="'+href+'" class="_text">'+maketext(el.text)+u'</a></div>'
    else:
      content += u'<a href="'+href+'" class="_editlink">&#x270e;</a></div>'
    content += style.apply( el, props )
    content += u'</div>\n'



class BlockEdit(Type):
  def getContent( self, el, style, props, content):
    content += u'<a name="f"></a>'
    content += u'<div class="'+makeattr(el.tag)+u'''"><div class="_edit">
<input type="hidden" name="location" value="'''+Type.getLocation( props )+'''" />
<input type="hidden" name="action" value="replace" />
<input class="_full" id="focus" name="b" value="'''
    if el.text:
      content += makeattr(el.text)
    content += u'" /><input type="hidden" name="a" value="'
    if el.text:
      content += makeattr(el.text)
    content += u'" /><button type="submit">Save</button></div></div>\n'

  def getScript( self, el, props ):
    return Type.editInit;



class Flag(Type):
  def getContent( self, el, style, props, content ):
    href = Type.getHref( props )

    content += u'<div class="'+makeattr(el.tag)+u'"><h4>'+maketext(el.tag)+u'</h4><div class="_">'
    if el.text and el.text.strip():
      content += u'<a href="'+href+'" class="_text">'+maketext(el.text)+u'</a></div>'
    else:
      content += u'<a href="'+href+'" class="_editlink">&#x270e;</a></div>'
    content += style.apply( el, props )
    content += u'</div>\n'

  def getStyle( self, el, props ):
    selector = self.getSelector( el, props )

    colour = self.options.get( 'flagColour', Type.getColour( el.tag ) )

    sideStyle = 'border-left: 5px solid '+colour+'; border-bottom: 5px solid transparent; margin-left: 5px;'
    sideStyle += self.options.get( 'style', '' )
    flagStyle = 'color: '+colour+'; font-size: 10px; padding: 0 0 0 0.5em;'
    flagStyle += self.options.get( 'flagStyle', '' )
    logger.log(3,selector,sideStyle)
    logger.log(3,selector+'>h4',flagStyle)
    return selector, sideStyle, selector+'>h4', flagStyle

  def getScript( self, el, props ):
    return ''



class FlagEdit(Type):
  def getContent( self, el, style, props, content ):
    content += u'<a name="f"></a>'
    content += u'<div class="'+makeattr(el.tag)+u'">'
    content += u'<h4>'+maketext(el.tag)+u'</h4>'
    content += u'''<div class="_edit">
<input type="hidden" name="location" value="'''+Type.getLocation( props )+'''" />
<input type="hidden" name="action" value="replace" />
<input id="focus" name="b" value="'''
    if el.text:
      content += makeattr(el.text)
    content += u'" /><input type="hidden" name="a" value="'
    if el.text:
      content += makeattr(el.text)
    content += u'" /><button type="submit">Save</button></div></div>\n'

  def getStyle( self, el, props ):
    selector = self.getSelector( el, props )

    colour = self.options.get( 'flagColour', Type.getColour( el.tag ) )

    sideStyle = 'border-left: 5px solid '+colour+'; border-bottom: 5px solid transparent; margin-left: 5px;'
    sideStyle += self.options.get( 'style', '' )
    flagStyle = 'color: '+colour+'; font-size: 10px; padding: 0 0 0 0.5em;'
    flagStyle += self.options.get( 'flagStyle', '' )
    return selector, sideStyle, selector+'>h4', flagStyle

  def getScript( self, el, props ):
    return Type.editInit;



class BlockSplit(Type):
  def getContent( self, el, style, props, content):
    content += u'<a name="f"></a>'
    content += u'<div class="'+makeattr(el.tag)+u'''"><div class="_edit">
<input type="hidden" name="location" value="'''+Type.getLocation( props )+'''" />
<input type="hidden" name="action" value="addbefore" />
<input type="hidden" name="a" value="'''+makeattr(el.tag)+'''" />
<input type="hidden" name="validate" value="[\w]+" />
<input type="hidden" name="action_2" value="addbefore" />
<input type="hidden" name="a_2" value="'''+makeattr(el.tag)+'''" />
<input type="hidden" id="insert" name="b_2" value="" />
<input type="hidden" name="action_3" value="delete" />
<input type="hidden" name="a" value="'''+makeattr(el.tag)+'''" />
<input type="hidden" name="action_4" value="editafter" />
<input type="hidden" name="a" value="'''+makeattr(el.tag)+'''" />
<input class="_full" id="focus" name="b" value="'''
    if el.text:
      content += makeattr(el.text)
    content += u'" /><input type="hidden" name="b_3" value="'
    if el.text:
      content += makeattr(el.text)
    content += u'" /><button type="submit">Save</button></div></div>\n'

  def getScript( self, el, props ):
    return '''init = function() {
  if ( window.scrollY < window.scrollMaxY ) {
    window.scrollBy( 0, -window.innerHeight/3 );
  }
  thefocus = document.getElementById('focus');
  thefocus.onkeydown = handleKeyDown;
  thefocus.focus();
  thefocus.selectionStart = 0;
}

handleKeyDown = function( event ) {
  if ( event.keyCode == 13 ) {
    text = this.value;
    this.value = text.substring( 0, this.selectionStart );
    document.getElementById( 'insert' ).value = text.substring( this.selectionStart );
  }
}'''
