#!/usr/bin/python
#   coding: utf-8

import datetime
import time
import pad.helper
import pad.view
from pad.logger import Logger
from pad.content import Content
logger = Logger()


def jsDecoration( form, style, script, thetime=0 ):
  return script+'''

var populate = function( node ) {
  node.innerHTML="'''+form.replace('"','\\"').replace('\n','\\n')+'''";
  return node;
}

// '''+str( int( (time.time()-thetime)*1000 ) )+''' ms
'''

class Ajax(pad.view.View):
  def getResponse( self, renderResult ):
    form, style, script = renderResult
    return 'application/javascript', jsDecoration( form, style, script, time.time() )






def defaultDecoration( form, style, script, thetime=0 ):
  return '''<html>
<head>
  <link rel="stylesheet" type="text/css" href="/static/default.css" />
  <!--link rel="stylesheet" type="text/css" href="/static/dojo/dojo/resources/dojo.css" /-->
  <!--script type="text/javascript" src="/static/dojo/dojo/dojo.js"
          djConfig="parseOnLoad:true, isDebug:true"></script-->
  <script type="text/javascript">dojo.require("dijit.form.Form");
  '''+script+'''
  </script>
  <style type="text/css">
  '''+style+'''
  </style>
</head>
<body onload="init()">
'''+form+'''
<div class="_debug"><p>'''+str( int( (time.time()-thetime)*1000 ) )+''' ms</p></div>
</body>
</html>
'''


class Single(pad.view.View):
  def serve( self, name, query='' ):
    thetime=time.time()
    return None, defaultDecoration( *(self.apply( *self.parseName( name, query ) ) + (thetime,) ) )
    


class Blog(pad.view.View):
  def serve( self, name, query='' ):
    thetime=time.time()

    ns, path, props = self.parseName( name, query )

    if path == '':
      return 'seeother', datetime.datetime.now().isoformat()

    return None, defaultDecoration( *(self.apply( *self.parseName( name, query ) ) + (thetime,) ) )


  def apply( self, ns, path, props ):
    try:
      posts = int(props['posts'])
      if posts > self.options.get( 'maxPosts', 50 ):
        posts = maxPosts
    except (ValueError, KeyError):
      posts = self.options.get( 'posts', 5 )

    logger.log(3, 'Looking for ',posts,' posts before ',path)
    names = self.storage.listPrecedingSiblings( ns, path, posts )
    logger.log(3, 'Found ', names)

    form  = '<form method="POST" accept-charset="utf-8" dojoType="dijit.form.Form" id="form">\n'

    if props.has_key('location'):
      form += '<input type="hidden" name="path" value="'+pad.helper.makeattr(props['path'][0])+'" />\n'
      for item in props['location']:
        form+='<input type="hidden" name="location" value="'+item+'" />\n'

    content = Content()

    for index,name in enumerate(names):
      nameprops = props.copy()
      nameprops['path'] = name
      parent = '_page_'+str(index)
      nameprops['parent']= parent

      model = self.getModel( ns, name )
      tree = self.storage.get( ns, name )

      if tree is None:
        tree = self.getTemplate( ns, name, nameprops, model )

      if tree.dtd:
        logger.log(3,ns,'/',name,': overriding model with DTD ',tree.dtd)
        model = self.models[tree.dtd]

      logger.log(3,ns,'/',name,': using model ',model)

      if tree is None:
        if ns==self.ns[None]:
          # know it exists so must be another problem
          raise IOError('Problem reading file '+name)
        else:
          tree = self.getRemote( ns, name )

      logger.log(3,' Got tree (', tree.tree.tag, ')' )

      if props.has_key('path') and name==props['path'][0]:
        form += '<input type="hidden" name="version" value="'+str(tree.version)+'" />\n'
      else:
        if nameprops.has_key('location'):
          del(nameprops['location'])

      logger.log(3, 'Blog view: using props ', nameprops )

      newProps = model.getProperties( tree.tree )
      logger.log(3,'Found properties ', newProps)

      nameprops.update( newProps )

      content += '<div class="'+parent+'">'
      content = model.getContent( tree, self.styleName, nameprops, content )
      content += '</div>'

    form += content.getContent() +'</form>'
    style = '<style type="text/css">\n'+content.getStyles()+'</style>\n'
    script = '<script lang="javascript">\n'+content.getScripts()+'</script>\n'

    return form, style, script
