#!/usr/bin/python
#   coding: utf-8

import re
import cStringIO
import pad.model
import pad.validator
import pad.setter
import xml.etree.cElementTree as etree

from pad.logger import Logger
logger = Logger()

padns='{http://pad.colourcountry.net/xmlns/model}'

def new(className, objargs, **aargs):
  ns = aargs.get( 'namespace', globals() )
  if '.' in className:
    theModule, theRest = className.split('.',1)
    return new(theRest, objargs, namespace = ns[theModule].__dict__)
  logger.log( 3, 'New ',className,' ',objargs )
  return ns[className](**objargs)

def getArgs( el ):
  args = {}
  for child in el:
    if child.tag == padns+'string':
      args[child.get('name')] = child.get('value', '')
    elif child.tag == padns+'list':
      args[child.get('name')] = getArgs( child )
    else:
      raise ValueError( 'Unknown tag '+child.tag )
  return args

class XmlModel(pad.model.Model):
  def __init__( self, filename ):
    content = file( filename, 'r' ).read()
    parser = etree.XMLParser()
    parser.feed( content )
    tree = parser.close()

    logger.log(3, 'Creating model for ', tree.tag )
    assert tree.tag.startswith(padns)

    editModes = {}
    validModes = {}
    setterDef = {}

    for element in tree.find( padns+'elements').findall( padns+'element' ):
      elementName = element.get( 'name', None )
      logger.log(3, 'Building element ', elementName, '...' )

      setter = element.find( padns+'setter' )
      if setter:
        args = getArgs( setter )
        setterDef[elementName] = new( setter.get('name'), args )

      for mode in element.findall( padns+'mode' ):
        modeName = mode.get( 'name', None )

        if not editModes.has_key( modeName ):
          editModes[modeName] = {}
        if not editModes[modeName].has_key( elementName ):
          editModes[modeName][elementName] = []
        editStyles = []

        if not validModes.has_key( modeName ):
          validModes[modeName] = {}
        if not validModes[modeName].has_key( elementName ):
          validModes[modeName][elementName] = []
        validStyles = []

        for style in mode.findall( padns+'style' ):
          tests = []

          for test in style.findall( padns+'test' ):
            tests.append( test.get( 'property', None ) )
            tests.append( test.get( 'value', None ) )

          editor = style.find( padns+'editor' )

          if editor is not None:
            editStyles.append(tests)
            args = getArgs( editor )
            editStyles.append( new( editor.get('name'), args ) )

          validator = style.find( padns+'validator' )

          if validator is not None:
            validStyles.append(tests)
            args = getArgs( validator )
            validStyles.append( new( validator.get('name'), args ) )

        editModes[modeName][elementName] = editStyles
        validModes[modeName][elementName] = validStyles
        logger.log(3, 'Added mode ', modeName, '.' )
      logger.log(3, 'Finished element ', elementName, '.' )


    self.styles = {}
    for mode in editModes.keys():
      logger.log(3, mode, ' (edit) = ', editModes[mode])
      self.styles[mode] = pad.style.Style( editModes[mode] )

    self.validators = {}
    for mode in validModes.keys():
      logger.log(3, mode, ' (valid) = ', validModes[mode])
      self.validators[mode] = pad.validator.ByTag( validModes[mode] )

    self.setter = setterDef

    template = tree.find( padns+'template' )[0]

    crappyApi = cStringIO.StringIO()
    etree.ElementTree( template ).write( crappyApi )
    crappyApi.reset()
    self.template = crappyApi.read()

    logger.log(3, 'template = ', self.template)



if __name__ == "__main__":
  model = XmlModel('pagedef.xml')
