#!/usr/bin/python
#   coding: utf-8

import re
import urllib

def maketext( text ):
  return text.replace( '&', '&amp;' ).replace( '<', '&lt;' ).replace( '>', '&gt;' )

def makeattr( text ):
  return maketext(text).replace( '"', '&quot;' )

def makejsname( text ):
  return urllib.quote_plus(text).replace( '+', '_' ).replace( '%', '_' ).replace('.', '_')

def makecssname( text ):
  return re.sub('([^a-zA-Z_0-9-])','\\\\\\1',text)

def makeNameAndAttrs( el ):
  unqualifiedTag = re.sub('{[^}]+}','',el.tag)
  attrs=''
  for attr in el.keys():
    attrs+=' '+attr+'='+el.get(attr)
  return maketext( unqualifiedTag + attrs )

def makejsobj( thedict ):
  assert isinstance( thedict, dict )
  obj = '{'
  prefix = ''
  for key in thedict:
    if re.search('[^A-Za-z_]', key) or key.startswith('_'):
      pass # do not format invalid or private key
    else:
      value = str(thedict[key]).replace('\\','\\\\').replace('"','\\"').replace('\n','\\n')
      obj += prefix+key+':"'+value+'"'
    prefix = ','
  obj += '}'
  return obj


def formatElement( el ):
  tag = el.tag
  attrs = u''
  for attr in el.keys():
    attrs+= ' '+attr+'="'+makeattr( el.get(attr) )+'"'
  if el.text or len(el)>0:
    s = u'<'+tag+attrs+'>'+formatChildren( el )+'</'+tag+'>'
  else:
    s = u'<'+tag+attrs+' />'
  return s

def formatChildren( el ):
  s = u''
  if el.text: s += maketext( el.text )
  for child in el:
    s+= formatElement( child )
    if child.tail: s += maketext( child.tail )
  return s

def encodeElement( el ):
  return formatElement(el).encode('utf-8')

def encodeChildren( el ):
  return formatChildren(el).encode('utf-8')




if __name__=='__main__':
  testString = '{http://x.net/}c\\"a-'+"shan't"+'<p>'

  print testString
  print maketext(testString)
  print makeattr(testString)
  print makejsname(testString)
  print makecssname(testString)

