#!/usr/bin/python
#   coding: utf-8

import os, re
import xml.etree.cElementTree as etree
import xml.etree.ElementPath as epath
import pad.helper
from pad.logger import Logger
logger = Logger()


class SimpleTree:
  def __init__( self, content ):
    logger.log(3, 'Building simple tree from ',content)
    parser = etree.XMLParser()
    parser.feed( content.encode('utf-8') )
    self.tree = parser.close()

  def __iter__( self ):
    yield self.tree
    raise StopIteration

  def getPath( self, path ):
    result = epath.find( self.tree, path )
    logger.log(3, 'Looking for ',path,' in ',self.tree.tag,':',result)
    return result

  def getXML( self ):
    return pad.helper.formatElement(self.tree).encode('utf-8')

  def __repr__( self ):
    def formatElement( el, tab='- ' ):
      s = tab+el.tag
      if el.text: s += ' '+el.text
      s += '\n'
      for child in el:
        s+= formatElement( child, '  '+tab )
      return s

    s = '(Tree from {'+self.ns+'}'+self.path+', version '+self.version+')\n'
    s += formatElement(self.tree)
    return s  

  def replaceValues( self, values ):
    def rv( el, values ):
      for index in range( len(el)-1, -1, -1 ):
        child = el[index]
        logger.log(3, child.tag )
        if child.tag == '{http://pad.colourcountry.net/xmlns/model}value-of':
          newtext = values.get(child.get('property'),'')
          logger.log(3, 'Replacing value-of ',child.get('property'),' with property value ',newtext )
          if child.tail:
            newtext += child.tail
          if index==0:
            if el.text:
              el.text += newtext
            else:
              el.text = newtext
          else:
            if el[index-1].tail:
              el[index-1].tail += newtext
            else:
              el[index-1].tail = newtext
          del( el[index] )
        else:
          rv( child, values )

    rv( self.tree, values )


class Tree(SimpleTree):
  def __init__(	self, content, version, ns, path, isReadOnly=False ):
    self.content = content
    self.version = unicode(version)
    self.ns = ns
    self.path = path
    self.isReadOnly = isReadOnly

    parser = etree.XMLParser()
    parser.feed( content )
    self.tree = parser.close()

    doctype = re.search('<\!DOCTYPE ([^ ]+) (SYSTEM|PUBLIC) "([^"]+)"',content)
    if doctype:
      self.doctypetype = doctype.group(2)
      if self.doctypetype=='PUBLIC':
        ## someone still using public identifiers, how sweet
        self.dtd = 'public:'+doctype.group(3)
      else:
        self.dtd = doctype.group(3)
    else:
      ## TODO: support schemas
      ## for the moment just guess that we can find the model at the root level
      self.doctypetype= 'SYSTEM'
      self.dtd = 'pad://'+ns+'/'+self.tree.tag

  def getXML( self ):
    s = '<!DOCTYPE '+self.tree.tag.encode('utf-8')+' '+self.doctypetype+' "'+self.dtd+'" >'
    s += pad.helper.formatElement(self.tree).encode('utf-8')
    return s

  def getReplace( self, location, newContent ):
    x=self.tree
    for index in location[1:]:
      logger.log(3, x, index)
      x = x[index]

    oldContent = pad.helper.formatChildren( x )
    if oldContent == newContent:
      return None
    else:
      return Replace( location, self.version, oldContent, newContent )

  def getDelete( self, location ):
    x=self.tree
    for index in location[1:]:
      logger.log(3, x, index)
      x = x[index]

    oldContent = pad.helper.formatChildren( x )
    return Delete( location, self.version, x.tag, oldContent )

  def getAddBefore( self, location, newTag, newContent ):
    return AddBefore( location, self.version, newTag, newContent )

