#!/usr/bin/python
#   coding: utf-8

import re
import time
import urllib

import pad.helper
import pad.tree
from pad.logger import Logger
logger = Logger()

def splitQuery( query ):
  queries={}
  if query.startswith('?'):
    query=query[1:]

  if query:
    for item in query.split( '&' ):
      try:
        k, v = item.split( '=' )
        if queries.has_key(k):
          queries[k].append(re.sub('[^0-9A-Za-z,]','',urllib.unquote_plus(v)))
        else:
          queries[k]=[urllib.unquote_plus(v)]
      except IndexError:
        pass    

  return queries

def buildQuery( d ):
  pairs = []
  for key in d.keys():
    pairs.append(urllib.quote_plus(key)+"="+urllib.quote_plus(d[key]))
  return '?'+'&'.join(pairs)+'#f'

def parseLocation( s ):
  location = []
  for item in s.split(','):
    try:
      location.append( int(item) )
    except ValueError:
      location.append( item )
  return tuple( location )

class View:
  mimetype = 'text/html'
  charset = 'utf-8'

  def __init__(	self, href, styleName, ns=None, models=None, storage=None, **aargs ):
    self.storage = storage
    self.models = models
    self.href = href
    self.styleName = styleName
    self.ns = { None:ns }
    self.options = aargs
    

  def parseName( self, name, query='' ):
    try:
      ns, path = re.split( '(?<!/)/(?!/)', name, 1 )
    except ValueError:
      raise NotImplementedError

    ns = ns.replace('+','/')

    props = {}
    props.update( splitQuery(query) )
    props['ns'] = ns
    props['path'] = path

    try:
      props['basename'] = path[path.rindex('/')+1:]
    except ValueError:
      props['basename'] = path

    logger.log(2, 'Request for ', ns, ':', path, props)

    return ns, path, props

  def getTemplate( self, ns, path, props, model=None ):
    if model is None:
      model = self.models.getByPath( ns, path )
      logger.log(3, 'Getting template from model ',model)
    
    return model.getTemplate( ns, path, props )


  def renderPath( self, ns, path, props={} ):
    tree = self.storage.get( ns, path )

    if tree is None:
      tree = self.getTemplate( ns, path, props )

    logger.log(3,'renderPath: ',ns,'/',path)

    if tree is None:
      if ns==self.ns[None]:
        tree = self.getNotFound( ns, path )
      else:
        tree = self.getRemote( ns, path )

    return self.renderTree( tree, props )


  def renderTree( self, tree, props={} ):

    logger.log(3,'renderTree: using DTD ',tree.dtd)
    model = self.models[tree.dtd]
    logger.log(3,'renderTree: using model ',model)

    logger.log(3,' Got tree (', tree.tree.tag, ')' )

    newProps = model.getProperties( tree.tree )
    logger.log(3,'Found properties ', newProps)

    props.update( newProps )

    props["version"] = tree.version
    props["dtd"] = tree.dtd

    if tree.isReadOnly:
      props["read-only"] = "true"

    if props.has_key("style"):
      if isinstance( props["style"], list ):
        styleName = props["style"][0]
      else:
        styleName = props["style"]
    else:
      styleName = self.styleName

    content = model.getContent( tree.tree, styleName, props )

    return self.renderContent( content, props )

  def update( self, name, query={} ):
    logger.log(3, 'Update name: ', name)
    logger.log(3, 'Update query: ', query)

    try:
      ns, path = re.split( '(?<!/)/(?!/)', name, 1 )
    except ValueError:
      raise NotImplementedError

    ns = ns.replace('+','/') #FIXME

    attrs = {}

    # apply
    # location_x + action_x + a_x + b_x then
    # location_y + action_y + a_y + b_y then
    for attr in query.keys():
      try:
        attrbase, attrkey = attr.split('_',1)
        if attrs.has_key(attrkey):
          attrs[attrkey][attrbase] = query.pop(attr).decode('utf-8')
        else:
          attrs[attrkey] = { attrbase: query.pop(attr).decode('utf-8') }
      except ValueError:
        # just a plain query field
        query[attr] = query[attr].decode('utf-8') 

    logger.log(3, 'Update attributes: ', attrs)

    attrkeys = [None] + attrs.keys()
    attrkeys.sort()

    deltas = []
    valid = True
    returnValue = ( 'text/plain', '' )

    for key in attrkeys:
          try:
            action = attrs[key]['action']
          except KeyError:
            action = query.pop('action')

          try:
            actpath = attrs[key]['ns']
          except KeyError:
            try:
              actns = query.pop('ns')
            except KeyError:
              actns = ns

          try:
            actpath = attrs[key]['path']
          except KeyError:
            try:
              actpath = query.pop('path')
            except KeyError:
              actpath = path

          try:
            location = attrs[key]['location']
          except KeyError:
            location = query.pop('location')

          location = parseLocation( location )

          if action=='editafter':
            # Note: this action does not range check location
            newLocation = location[:-1] + ( location[-1]+1 ,)
            newLocation = ','.join( [str(x) for x in newLocation] )

          elif action=='editbefore':
            # Note: this action does not range check location
            newLocation = location[:-1] + ( location[-1]+1 ,) 
            newLocation = ','.join( [str(x) for x in newLocation] )

          try:
            version = attrs[key]['version']
          except KeyError:
            version = query.pop('version')

          try:
            a = attrs[key]['a']
          except KeyError:
            a = query.pop('a')

          try:
            b = attrs[key]['b']
          except KeyError:
            b = query.pop('b')


          # To do: validation

          logger.log(3, 'Applying action:', action, ', ', location, ', ', version, ', ', a, ', ', b )

          delta = self.storage.getDelta( action, location, version, a, b )
          if delta is not None:
            deltas.append( (actns, actpath, self.storage.getDelta( action, location, version, a, b ) ) )

    logger.log(3, 'Remaining query fields: ', query)
            
    if valid:
      logger.log(3, 'Finished updating with result', returnValue )
      conflict = self.applyDeltas( deltas )
      if conflict:
        # format conflict information as form result - as not yet saved
        return self.getResponse( self.renderTree( conflict, query ) )
      else:
        # page is valid so redirect to edited page
        # pass through any query fields that we didn't know what to do with
        # query['location'] = actlocation # this was here but is dubious as doesn't take into account multiple edits

        return ('seeother', self.href+'/'+urllib.quote_plus(actns)+'/'+urllib.quote(actpath)+buildQuery( query ) )
    else:
      return ( 'error', 'Failed validation' )


  def applyDeltas( self, deltas ):
    if len(deltas)==0:
      logger.log(1, 'No deltas found in submission')
      return None

    for actns, actpath, delta in deltas:
      logger.log(3, 'Applying delta:', delta )
      conflict = self.storage.apply( actns, actpath, delta, lambda: self.getTemplate(actns, actpath, {}) )
      if conflict:
        logger.log(3, 'Conflict detected on ',actns,'/',actpath)
        logger.log(4, 'Using tree: ', conflict.getXML() )

    # FIXME: will only report conflicts on one ns/path
    # all other attempted edits will be silently discarded
    return conflict

  def serve( self, name, query='' ):
    return self.getResponse( self.renderPath( *self.parseName(name, query) ) )

  def getResponse( self, renderResult ):
    return self.__class__.mimetype+'; charset='+self.__class__.charset, renderResult

  def submit( self, name, query='' ):
    return self.update( name, query )

  def getNotFound( self, ns, path ):
    raise NotImplementedError( "Not found and no template" )

  def getRemote( self, ns, path ):
    raise NotImplementedError( "Remote file" )

