#!/usr/bin/python
#   coding: utf-8

import re
import datetime
import time
import pad.helper
from pad.view import View
from pad.logger import Logger
logger = Logger()


class Ajax(View):
  mimetype = 'application/javascript'

  def renderContent( self, content, props={} ):
    def jsProtect(s):
      return s.replace('"','\\"').replace('\n','\\n')

    logger.log(3, 'Rendering', content)

    viewport = '<div class="_viewport">'+content.concat()+'</div>'

    padprops = pad.helper.makejsobj(props)

    return '''({
  requires: ['''+','.join(content.get('requires'))+'''],

  populate: function( node ) {
    node.innerHTML="'''+jsProtect(viewport)+'''";
    node.firstChild.padProps='''+padprops+''';
    return node;
  }
})
'''
    






def defaultDecoration( form, style, script, thetime=0 ):
  return '''<html>
<head>
  <link rel="stylesheet" type="text/css" href="/static/default.css" />
  <script type="text/javascript">
  '''+script+'''
  </script>
  <style type="text/css">
  '''+style+'''
  </style>
</head>
<body onload="init()">
'''+form+'''
<div class="_debug"><p>'''+str( int( (time.time()-thetime)*1000 ) )+''' ms</p></div>
</body>
</html>
'''


class HTML(View):
  def serve( self, name, query='' ):
    thetime=time.time()
    return None, defaultDecoration( *(self.apply( *self.parseName( name, query ) ) + (thetime,) ) )

  def renderContent( self, content ):
    form  = '''
 <form name="tree-info">
  <input type="hidden" name="version" value="'''+content['version']+'''" />
  <input type="hidden" name="ns" value="'''+content['ns']+'''" />
  <input type="hidden" name="path" value="'''+content['path']+'''" />
</form>
'''
    form += '<div class="_viewport">'+form+content[None]+'</div>'
    return form    



class BlogHTML(HTML):
  def serve( self, name, query='' ):
    thetime=time.time()

    ns, path, props = self.parseName( name, query )

    if path == '':
      return 'seeother', datetime.datetime.now().isoformat()

    return None, defaultDecoration( *(self.apply( *self.parseName( name, query ) ) + (thetime,) ) )


  def apply( self, ns, path, props ):
    try:
      posts = int(props['posts'])
      if posts > self.options.get( 'maxPosts', 50 ):
        posts = maxPosts
    except (ValueError, KeyError):
      posts = self.options.get( 'posts', 5 )

    logger.log(3, 'Looking for ',posts,' posts before ',path)
    names = self.storage.listPrecedingSiblings( ns, path, posts )
    logger.log(3, 'Found ', names)

    form  = '<form method="POST" accept-charset="utf-8" dojoType="dijit.form.Form" id="form">\n'

    if props.has_key('location'):
      form += '<input type="hidden" name="path" value="'+pad.helper.makeattr(props['path'][0])+'" />\n'
      for item in props['location']:
        form+='<input type="hidden" name="location" value="'+item+'" />\n'

    content = Content()

    for index,name in enumerate(names):
      nameprops = props.copy()
      nameprops['path'] = name
      parent = '_page_'+str(index)
      nameprops['parent']= parent

      model = self.getModel( ns, name )
      tree = self.storage.get( ns, name )

      if tree is None:
        tree = self.getTemplate( ns, name, nameprops, model )

      if tree.dtd:
        logger.log(3,ns,'/',name,': overriding model with DTD ',tree.dtd)
        model = self.models[tree.dtd]

      logger.log(3,ns,'/',name,': using model ',model)

      if tree is None:
        if ns==self.ns[None]:
          # know it exists so must be another problem
          raise IOError('Problem reading file '+name)
        else:
          tree = self.getRemote( ns, name )

      logger.log(3,' Got tree (', tree.tree.tag, ')' )

      if props.has_key('path') and name==props['path'][0]:
        form += '<input type="hidden" name="version" value="'+str(tree.version)+'" />\n'
      else:
        if nameprops.has_key('location'):
          del(nameprops['location'])

      logger.log(3, 'Blog view: using props ', nameprops )

      newProps = model.getProperties( tree.tree )
      logger.log(3,'Found properties ', newProps)

      nameprops.update( newProps )

      content += '<div class="'+parent+'">'
      content = model.getContent( tree, self.styleName, nameprops, content )
      content += '</div>'

    form += content.getContent() +'</form>'
    style = '<style type="text/css">\n'+content.getStyles()+'</style>\n'
    script = '<script lang="javascript">\n'+content.getScripts()+'</script>\n'

    return form, style, script
