// pad.js v1.9

PAD = {
	utils: {},
	state: {},
	modules: {},
	callbacks: {},
	actions: {},
	events: [],
}

PAD.actions.registerEvent = function( ns, path, callback ) {
  PAD.events.push( { ns:ns, path:path, callback:callback } );
}

PAD.actions.fireEvent = function( ns, path, reason ) {
  for (var i=0, a; a=PAD.events[i++];) {
    if (a.ns==ns && a.path==path) {
      a.callback( reason );
    }
  }
}

PAD.actions.deregisterEvent = function( ns, path, callback ) {
  for (var i=0, a; a=PAD.events[i++];) {
    if (a.ns==ns && a.path==path && a.callback==callback) {
      PAD.events.splice(i,1);
      return;
    }
  } 
}

PAD.utils.dumpObject = function(obj, message) {
  if (!message) { message = obj; }
  var details = "*****************" + "\n" + message + "\n";
  var fieldContents;
  for (var field in obj) {
    fieldContents = obj[field];
    if (typeof(fieldContents) == "function") {
      fieldContents = "(function)";
    }
    details += "  " + field + ": " + fieldContents + "\n";
  }
  console.log(details);
}

PAD.utils.verify = function( module ) {
  if ( PAD.modules[module] == null ) {
    PAD.actions.loadModule( module );
  }
}

PAD.actions.loadModule = function( module ) {
  PAD.utils.sendRequest( '/static/modules/'+module+'.js', 'GET', function( json ) { PAD.callbacks.loadModule( json, module ) } );
};

PAD.callbacks.loadModule = function( json, module ) {
  PAD.modules[module] = eval(json);
};

PAD.utils.findForm = function( node ) {
  if (node == null) {
    return null;
  } else if (node.parentNode.tagName == "FORM" ) {
    return node.parentNode;
  } else if (node.parentNode === document) {
    return null;
  } else {
    return PAD.utils.findForm(node.parentNode);
  }
}

PAD.utils.findProps = function( node ) {
  if (node == null) {
    return null;
  } else if (typeof node.parentNode.padProps != "undefined" ) {
    props = node.parentNode.padProps;

    // save the node we found the props in
    props._viewport = node.parentNode;

    return props;

  } else if (node.parentNode === document) {
    return null;
  } else {
    return PAD.utils.findProps(node.parentNode);
  }
}

PAD.utils.doNothing = function() {};

PAD.utils.sendRequest = function( uri, method, successCallback ) {
   var req = new XMLHttpRequest();  
   req.open(method, uri, true);  
   req.onreadystatechange = function (aEvt) {  
     if (req.readyState == 4) {  
       if (req.status == 200 && successCallback != null) {
         successCallback( req.responseText );  
       } else {  
       };
     };
   };  
   req.send(null);  
};

PAD.state.forceDecision = null;
PAD.state.activeNode = null;
PAD.state.deactivate = PAD.utils.doNothing;

PAD.callbacks.populatePage = function( json, element ) {
  var response = eval( json );

  if ( response.requires != null ) {
    for ( var i=0, len = response.requires.length ; i < len ; i++ ) {
      PAD.utils.verify( response.requires[i] );
    }
  }

  if ( response.populate != null ) {
    if ( element == null ) {
      // panic
      element = document.getElementsByTagName("BODY")[0];
    }
    response.populate( element );
  }
}

PAD.callbacks.replacePage = function( json, content ) {
  var newcontent = document.createElement("div");

  PAD.callbacks.populatePage( json, newcontent );
  content.parentNode.replaceChild( newcontent, content );
  return newcontent;
}


PAD.actions.openPage = function( ns, path, element ) {
  var request = '/pad/'+ns+'/'+path+'?style=ajax'

  var content = document.createElement("div");
  element.appendChild( content );

  PAD.actions.registerEvent( ns, path, function( json ) {
    content = PAD.callbacks.replacePage( json, content );
  });

  PAD.utils.sendRequest( '/pad/'+ns+'/'+path+'?style=ajax', 'GET', function( json ) {
    PAD.callbacks.populatePage( json, content );
  });

};

PAD.actions.openChildByTimestamp = function( ns, path, elementId ) {

  var zeropad = function(num, width) {
    num = num.toString();
    while (num.length < width)
    num = "0" + num;
    return num;
  }
  theDate = new Date();
  path += '/';
  path += theDate.getFullYear() + '-';
  path += zeropad( theDate.getMonth(),2 ) + '-';
  path += zeropad( theDate.getDate(),2 )+ ' at ';
  path += zeropad( theDate.getHours(),2 )+ ':';
  path += zeropad( theDate.getMinutes(),2 )+ ':';
  path += zeropad( theDate.getSeconds(),2 )

  PAD.actions.openPage( ns, path, elementId );
}

PAD.actions.submit = function( form ) {

  var props = PAD.utils.findProps( form );

  var query = '';
  for (var i=0, a; a=form.elements[i++];) {
    if (query != '') { query += '&' };
    query += a.name + '=' + encodeURIComponent( a.value );
  }
  query += '&style=ajax&version=' + encodeURIComponent( props.version );

  var url = '/pad/'+encodeURIComponent( props.ns )+'/'+encodeURI( props.path )+'?'+query;

  PAD.utils.sendRequest( url, 'POST', function( json ) {
    PAD.actions.fireEvent( props.ns, props.path, json );
  });

  PAD.state.deactivate();
  PAD.state.deactivate = PAD.utils.doNothing;

};
